package com.trs.ggzyexchange.schedul;


import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.fasterxml.jackson.core.JsonProcessingException;
import com.trs.ggzyexchange.entity.*;
import com.trs.ggzyexchange.enums.ClassIfy;
import com.trs.ggzyexchange.enums.JobLock;
import com.trs.ggzyexchange.service.*;
import com.trs.ggzyexchange.utils.TenderDetailsVoUtils;
import org.apache.commons.lang3.ObjectUtils;
import org.quartz.Job;
import org.quartz.JobExecutionContext;
import org.quartz.JobExecutionException;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

/**
 * <p>
 * 成交结果公告 定时任务
 * </p>
 *
 * @author trs
 * @since 2023-02-24
 */
@Component
public class ZrzySuppleNoticeSchedul implements Job {

    private Logger logger = LoggerFactory.getLogger(ZrzySuppleNoticeSchedul.class);

    @Autowired
    private ZrzySuppleService zrzySuppleService;

    @Autowired
    private GcNoticeandfileService gcNoticeandfileService;

    @Autowired
    private TenderDetailsVoService tenderDetailsVoService;

    @Autowired
    private ScheduledTashService scheduledTashService;

    @Autowired
    private AreaPlatformService areaPlatformService;

    public void queryPageListUpdateEs() throws JsonProcessingException {
        /**
         * 查询当前在定时任务表当中的最新时间
         */
        QueryWrapper<ScheduledTash> scheduledTashQueryWrapper = new QueryWrapper<ScheduledTash>();
        scheduledTashQueryWrapper.eq("table_name_value","zrzy_supple");
        Page<ScheduledTash> pageScheduledTash = new Page<ScheduledTash>(1, 1);
        Page<ScheduledTash> page = scheduledTashService.page(pageScheduledTash, scheduledTashQueryWrapper);
        List<ScheduledTash> records = page.getRecords();
        ScheduledTash scheduledTash = null;
        if(records.size() > 0){
            scheduledTash = records.get(0);
        }
        Long time = null;
        if(ObjectUtils.isEmpty(scheduledTash)){
            try {
                /**
                 * 如果scheduledTash为空，证明没有当前定时任务，设置当前的定时任务的时间为0  查询表gcNoticeandfile表当中的最新时间
                 */
                ScheduledTash scheduledTashTash = new ScheduledTash();
                scheduledTashTash.setTime(0L);
                time = 0L;
                scheduledTashTash.setTableNameValue("zrzy_supple");
                scheduledTashTash.setUpdateTime(new Date());
                scheduledTashTash.setCreateTime(new Date());
                scheduledTashService.saveOrUpdate(scheduledTashTash,scheduledTashQueryWrapper);
            }catch (Exception e){
                e.printStackTrace();
            }
        }else {
            /**
             * 如果scheduledTash为空，证明没有当前定时任务，设置当前的定时任务的时间为0  查询表gcNoticeandfile表当中的最新时间
             */
            time = scheduledTash.getTime();
        }
        try {
            //查询表gcNoticeandfile表当中的最新时间
            Long newTime = zrzySuppleService.maxDataTimeStamp();
            ScheduledTash scheduledTashTash = new ScheduledTash();
            scheduledTashTash.setTime(newTime);
            scheduledTashTash.setTableNameValue("zrzy_supple");
            scheduledTashTash.setUpdateTime(new Date());
            scheduledTashService.saveOrUpdate(scheduledTashTash,scheduledTashQueryWrapper);
        }catch (Exception e){
            e.printStackTrace();
        }
        if(!ObjectUtils.isEmpty(time)){
            QueryWrapper<ZrzySupple> gcPerformanceQueryWrapper = new QueryWrapper<ZrzySupple>();
            gcPerformanceQueryWrapper.gt("DATA_TIMESTAMP",time);
            int count = zrzySuppleService.count(gcPerformanceQueryWrapper);
            Integer pageSize = 1000;
            Integer pageNum = (int)Math.ceil((double)count/pageSize);
            for(int i = 1; i <= pageNum;i++){
                List<TenderDetailsVo> tenderDetailsVoListAll = getTenderDetailsVoListAll(i, pageSize,time);
                gcNoticeandfileService.parseTenderDetailsVo(tenderDetailsVoListAll);
            }
        }
    }
    public List<TenderDetailsVo> getTenderDetailsVoListAll(Integer pageNo,Integer pageSize,Long time){
        QueryWrapper<ZrzySupple> gcPerformanceQueryWrapper = new QueryWrapper<ZrzySupple>();
        gcPerformanceQueryWrapper.gt("DATA_TIMESTAMP",time);
        Page<ZrzySupple> page = new Page<ZrzySupple>(pageNo, pageSize);
        Page<ZrzySupple> page1 = zrzySuppleService.page(page,gcPerformanceQueryWrapper);
        List<ZrzySupple> records = page1.getRecords();
        List<TenderDetailsVo> tenderDetailsVos = new ArrayList<TenderDetailsVo>();
        for (ZrzySupple record : records) {
            TenderDetailsVo tenderDetailsVo = new TenderDetailsVo();
            String projectCode1 = record.getAnnouncementGuid();
            if(!ObjectUtils.isEmpty(projectCode1)){
                tenderDetailsVo.setProjectCode(projectCode1);
            }
            String noticeName = record.getAnnouncementTitle();
            if(!ObjectUtils.isEmpty(noticeName)){
                tenderDetailsVo.setNoticeName(noticeName);
            }
            String projectName = record.getLandProjectName();
            if(!ObjectUtils.isEmpty(projectName)){
                tenderDetailsVo.setProjectName(projectName);
            }
            String projectCode = record.getAnnouncementGuid();
            if(!ObjectUtils.isEmpty(projectCode)){
                tenderDetailsVo.setTenderProjectCode(projectCode);
            }
            String noticeContent = record.getAnnouncementContent();
            if(!ObjectUtils.isEmpty(noticeContent)){
                tenderDetailsVo.setNoticeContent(noticeContent);
            }

            Long noticeSendTime = record.getIssueTime();
            Long createDate = record.getCreateDate();
            if(false&&!ObjectUtils.isEmpty(createDate)&&0 != createDate){
                tenderDetailsVo.setNoticeSendTime(record.getCreateDate().toString());
            }else{
                if(!ObjectUtils.isEmpty(noticeSendTime)){
                    tenderDetailsVo.setNoticeSendTime(noticeSendTime.toString());
                    tenderDetailsVo.setDocGetStartTime(noticeSendTime.toString());
                }
            }


            String regionCode = record.getRegionCode();
            if(!ObjectUtils.isEmpty(regionCode)){
                tenderDetailsVo.setRegionCode(regionCode);
            }
            Long noticeEndTime = record.getIssueTime();
            if(!ObjectUtils.isEmpty(noticeEndTime)){
                tenderDetailsVo.setDocGetEndTime(noticeEndTime.toString());
            }
            String dataSourceCode = record.getSourceDataCode();
            if(!ObjectUtils.isEmpty(dataSourceCode)){
                tenderDetailsVo.setDataSourceCode(dataSourceCode);
            }
            String sourceDataKey = record.getSourceDataKey();
            if(!ObjectUtils.isEmpty(sourceDataKey)){
                tenderDetailsVo.setSourceDataKey(sourceDataKey+noticeSendTime);
            }

            if(!ObjectUtils.isEmpty(record.getTypeName())){
                tenderDetailsVo.setTypeName(record.getTypeName());
            }

            tenderDetailsVos.add(tenderDetailsVo);
        }
        for (TenderDetailsVo record : tenderDetailsVos) {
           try {
               record.setNoticeTypeCode(ClassIfy.SUPPLE_ANNOUNCEMENT.getCode());
               record.setNoticeTypeName(ClassIfy.SUPPLE_ANNOUNCEMENT.getName());
               record.setTransactionTypeCode(ClassIfy.NATURAL_RESOURCES.getCode());
               record.setTransactionTypeName(ClassIfy.NATURAL_RESOURCES.getName());
               record = tenderDetailsVoService.getTenderDetailsVoAllOrther(record);
               try {
                   String regionCode = record.getRegionCode();
                   AreaPlatform areaPlatformByRegionCode = areaPlatformService.getAreaPlatformByRegionCode(regionCode);
                   if(!ObjectUtils.isEmpty(areaPlatformByRegionCode)){
                       record.setPlatformCode(areaPlatformByRegionCode.getPlatformCode());
                       record.setPlatformName(areaPlatformByRegionCode.getRegionName());
                   }
                   String typeName = record.getTypeName();
                   switch (typeName){
                       case "TD":
                           record.setIndustriesTypeName(ClassIfy.zrzy_td.getName());
                           break;
                       case "KYQ":
                           record.setIndustriesTypeName(ClassIfy.zrzy_ky.getName());
                           break;
                       case "DK":
                           record.setIndustriesTypeName(ClassIfy.zrzy_dk.getName());
                           break;
                       case "BG":
                           record.setIndustriesTypeName(ClassIfy.zrzy_bg.getName());
                           break;
                       case "JY":
                           record.setIndustriesTypeName(ClassIfy.zrzy_jy.getName());
                           break;
                   }

               }catch (Exception e){
                   e.printStackTrace();
               }
               TenderDetailsVoUtils.isNull(record);
           }catch (Exception e){
               e.printStackTrace();
           }
        }
        return tenderDetailsVos;
    }

    @Override
    public void execute(JobExecutionContext jobExecutionContext) throws JobExecutionException {
        try {
            if(JobLock.ZrzySuppleFlag){
                JobLock.ZrzySuppleFlag = false;
                logger.info("执行ZrzySuppleFlag", new SimpleDateFormat().format(new Date()));
                try {
                    queryPageListUpdateEs();
                } catch (Exception e) {
                    e.printStackTrace();
                }
                JobLock.ZrzySuppleFlag = true;
            }else{
                logger.info("不执行ZrzySuppleFlag", new SimpleDateFormat().format(new Date()));
            }
        }catch (Exception e){
            e.printStackTrace();
            JobLock.ZrzySuppleFlag = true;
        }
    }

}
